package javax.naming;

import java.util.NoSuchElementException;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Properties;
import java.util.Vector;

class NameImpl
{
    private static final byte LEFT_TO_RIGHT = 1;
    private static final byte RIGHT_TO_LEFT = 2;
    private static final byte FLAT = 0;
    private Vector<String> components;
    private byte syntaxDirection;
    private String syntaxSeparator;
    private String syntaxSeparator2;
    private boolean syntaxCaseInsensitive;
    private boolean syntaxTrimBlanks;
    private String syntaxEscape;
    private String syntaxBeginQuote1;
    private String syntaxEndQuote1;
    private String syntaxBeginQuote2;
    private String syntaxEndQuote2;
    private String syntaxAvaSeparator;
    private String syntaxTypevalSeparator;
    private static final int STYLE_NONE = 0;
    private static final int STYLE_QUOTE1 = 1;
    private static final int STYLE_QUOTE2 = 2;
    private static final int STYLE_ESCAPE = 3;
    private int escapingStyle;
    
    private final boolean isA(String s, int toffset, String prefix) {
        return prefix != null && s.startsWith(prefix, toffset);
    }
    
    private final boolean isMeta(String s, int n) {
        return this.isA(s, n, this.syntaxEscape) || this.isA(s, n, this.syntaxBeginQuote1) || this.isA(s, n, this.syntaxBeginQuote2) || this.isSeparator(s, n);
    }
    
    private final boolean isSeparator(String s, int n) {
        return this.isA(s, n, this.syntaxSeparator) || this.isA(s, n, this.syntaxSeparator2);
    }
    
    private final int skipSeparator(String s, int n) {
        if (this.isA(s, n, this.syntaxSeparator)) {
            n += this.syntaxSeparator.length();
        }
        else if (this.isA(s, n, this.syntaxSeparator2)) {
            n += this.syntaxSeparator2.length();
        }
        return n;
    }
    
    private final int extractComp(String s, int i, int capacity, Vector<String> vector) throws InvalidNameException {
        int n = 1;
        StringBuffer sb = new StringBuffer(capacity);
        while (i < capacity) {
            int a;
            if (n != 0 && ((a = this.isA(s, i, this.syntaxBeginQuote1)) || this.isA(s, i, this.syntaxBeginQuote2))) {
                String s3 = a ? this.syntaxBeginQuote1 : this.syntaxBeginQuote2;
                String prefix = a ? this.syntaxEndQuote1 : this.syntaxEndQuote2;
                if (this.escapingStyle == 0) {
                    this.escapingStyle = (a ? 1 : 2);
                }
                for (i += s3.length(); i < capacity && !s.startsWith(prefix, i); ++i) {
                    if (this.isA(s, i, this.syntaxEscape) && this.isA(s, i + this.syntaxEscape.length(), prefix)) {
                        i += this.syntaxEscape.length();
                    }
                    sb.append(s.charAt(i));
                }
                if (i >= capacity) {
                    throw new InvalidNameException(s + ": no close quote");
                }
                i += prefix.length();
                if (i != capacity) {
                    if (this.isSeparator(s, i)) {
                        break;
                    }
                    throw new InvalidNameException(s + ": close quote appears before end of component");
                }
                else {
                    break;
                }
            }
            else {
                if (this.isSeparator(s, i)) {
                    break;
                }
                if (this.isA(s, i, this.syntaxEscape)) {
                    if (this.isMeta(s, i + this.syntaxEscape.length())) {
                        i += this.syntaxEscape.length();
                        if (this.escapingStyle == 0) {
                            this.escapingStyle = 3;
                        }
                    }
                    else if (i + this.syntaxEscape.length() >= capacity) {
                        throw new InvalidNameException(s + ": unescaped " + this.syntaxEscape + " at end of component");
                    }
                }
                else {
                    int a2;
                    if (this.isA(s, i, this.syntaxTypevalSeparator) && ((a2 = this.isA(s, i + this.syntaxTypevalSeparator.length(), this.syntaxBeginQuote1)) || this.isA(s, i + this.syntaxTypevalSeparator.length(), this.syntaxBeginQuote2))) {
                        String str = a2 ? this.syntaxBeginQuote1 : this.syntaxBeginQuote2;
                        String s2 = a2 ? this.syntaxEndQuote1 : this.syntaxEndQuote2;
                        i += this.syntaxTypevalSeparator.length();
                        sb.append(this.syntaxTypevalSeparator + str);
                        for (i += str.length(); i < capacity && !s.startsWith(s2, i); ++i) {
                            if (this.isA(s, i, this.syntaxEscape) && this.isA(s, i + this.syntaxEscape.length(), s2)) {
                                i += this.syntaxEscape.length();
                            }
                            sb.append(s.charAt(i));
                        }
                        if (i >= capacity) {
                            throw new InvalidNameException(s + ": typeval no close quote");
                        }
                        i += s2.length();
                        sb.append(s2);
                        if (i != capacity) {
                            if (this.isSeparator(s, i)) {
                                break;
                            }
                            throw new InvalidNameException(s.substring(i) + ": typeval close quote appears before end of component");
                        }
                        else {
                            break;
                        }
                    }
                }
                sb.append(s.charAt(i++));
                n = 0;
            }
        }
        if (this.syntaxDirection == 2) {
            vector.insertElementAt(sb.toString(), 0);
        }
        else {
            vector.addElement(sb.toString());
        }
        return i;
    }
    
    private static boolean getBoolean(Properties properties, String key) {
        return toBoolean(properties.getProperty(key));
    }
    
    private static boolean toBoolean(String s) {
        return s != null && s.toLowerCase(Locale.ENGLISH).equals("true");
    }
    
    private final void recordNamingConvention(Properties properties) {
        String property = properties.getProperty("jndi.syntax.direction", "flat");
        if (property.equals("left_to_right")) {
            this.syntaxDirection = 1;
        }
        else if (property.equals("right_to_left")) {
            this.syntaxDirection = 2;
        }
        else if (property.equals("flat")) {
            this.syntaxDirection = 0;
        }
        else {
            throw new IllegalArgumentException(property + "is not a valid value for the jndi.syntax.direction property");
        }
        if (this.syntaxDirection != 0) {
            this.syntaxSeparator = properties.getProperty("jndi.syntax.separator");
            this.syntaxSeparator2 = properties.getProperty("jndi.syntax.separator2");
            if (this.syntaxSeparator == null) {
                throw new IllegalArgumentException("jndi.syntax.separator property required for non-flat syntax");
            }
        }
        else {
            this.syntaxSeparator = null;
        }
        this.syntaxEscape = properties.getProperty("jndi.syntax.escape");
        this.syntaxCaseInsensitive = getBoolean(properties, "jndi.syntax.ignorecase");
        this.syntaxTrimBlanks = getBoolean(properties, "jndi.syntax.trimblanks");
        this.syntaxBeginQuote1 = properties.getProperty("jndi.syntax.beginquote");
        this.syntaxEndQuote1 = properties.getProperty("jndi.syntax.endquote");
        if (this.syntaxEndQuote1 == null && this.syntaxBeginQuote1 != null) {
            this.syntaxEndQuote1 = this.syntaxBeginQuote1;
        }
        else if (this.syntaxBeginQuote1 == null && this.syntaxEndQuote1 != null) {
            this.syntaxBeginQuote1 = this.syntaxEndQuote1;
        }
        this.syntaxBeginQuote2 = properties.getProperty("jndi.syntax.beginquote2");
        this.syntaxEndQuote2 = properties.getProperty("jndi.syntax.endquote2");
        if (this.syntaxEndQuote2 == null && this.syntaxBeginQuote2 != null) {
            this.syntaxEndQuote2 = this.syntaxBeginQuote2;
        }
        else if (this.syntaxBeginQuote2 == null && this.syntaxEndQuote2 != null) {
            this.syntaxBeginQuote2 = this.syntaxEndQuote2;
        }
        this.syntaxAvaSeparator = properties.getProperty("jndi.syntax.separator.ava");
        this.syntaxTypevalSeparator = properties.getProperty("jndi.syntax.separator.typeval");
    }
    
    NameImpl(Properties properties) {
        this.syntaxDirection = 1;
        this.syntaxSeparator = "/";
        this.syntaxSeparator2 = null;
        this.syntaxCaseInsensitive = false;
        this.syntaxTrimBlanks = false;
        this.syntaxEscape = "\\";
        this.syntaxBeginQuote1 = "\"";
        this.syntaxEndQuote1 = "\"";
        this.syntaxBeginQuote2 = "'";
        this.syntaxEndQuote2 = "'";
        this.syntaxAvaSeparator = null;
        this.syntaxTypevalSeparator = null;
        this.escapingStyle = 0;
        if (properties != null) {
            this.recordNamingConvention(properties);
        }
        this.components = new Vector();
    }
    
    NameImpl(Properties properties, String s) throws InvalidNameException {
        this(properties);
        int n = this.syntaxDirection == 2;
        boolean n2 = true;
        int length = s.length();
        int i = 0;
        while (i < length) {
            i = this.extractComp(s, i, length, this.components);
            if ((n ? ((String)this.components.firstElement()) : ((String)this.components.lastElement())).length() >= 1) {
                n2 = false;
            }
            if (i < length) {
                i = this.skipSeparator(s, i);
                if (i == length && n2 == 0) {
                    if (!(n == 0)) {
                        this.components.insertElementAt("", 0);
                    }
                    else {
                        this.components.addElement("");
                    }
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
    }
    
    NameImpl(Properties properties, Enumeration<String> enumeration) {
        this(properties);
        while (enumeration.hasMoreElements()) {
            this.components.addElement(enumeration.nextElement());
        }
    }
    
    private final String stringifyComp(String s) {
        int length = s.length();
        boolean n = false;
        boolean n2 = false;
        String str = null;
        String str2 = null;
        StringBuffer append = new StringBuffer(length);
        if (this.syntaxSeparator != null && s.indexOf(this.syntaxSeparator) >= 0) {
            if (this.syntaxBeginQuote1 != null) {
                str = this.syntaxBeginQuote1;
                str2 = this.syntaxEndQuote1;
            }
            else if (this.syntaxBeginQuote2 != null) {
                str = this.syntaxBeginQuote2;
                str2 = this.syntaxEndQuote2;
            }
            else if (this.syntaxEscape != null) {
                n = true;
            }
        }
        if (this.syntaxSeparator2 != null && s.indexOf(this.syntaxSeparator2) >= 0) {
            if (this.syntaxBeginQuote1 != null) {
                if (str == null) {
                    str = this.syntaxBeginQuote1;
                    str2 = this.syntaxEndQuote1;
                }
            }
            else if (this.syntaxBeginQuote2 != null) {
                if (str == null) {
                    str = this.syntaxBeginQuote2;
                    str2 = this.syntaxEndQuote2;
                }
            }
            else if (this.syntaxEscape != null) {
                n2 = true;
            }
        }
        if (str != null) {
            append = append.append(str);
            int j = 0;
            while (j < length) {
                if (s.startsWith(str2, j)) {
                    append.append(this.syntaxEscape).append(str2);
                    j += str2.length();
                }
                else {
                    append.append(s.charAt(j++));
                }
            }
            append.append(str2);
        }
        else {
            int n3 = 1;
            for (int i = 0; i < length; n3 = 0) {
                if (n3 != 0 && this.isA(s, i, this.syntaxBeginQuote1)) {
                    append.append(this.syntaxEscape).append(this.syntaxBeginQuote1);
                    i += this.syntaxBeginQuote1.length();
                }
                else if (n3 != 0 && this.isA(s, i, this.syntaxBeginQuote2)) {
                    append.append(this.syntaxEscape).append(this.syntaxBeginQuote2);
                    i += this.syntaxBeginQuote2.length();
                }
                else if (this.isA(s, i, this.syntaxEscape)) {
                    if (i + this.syntaxEscape.length() >= length) {
                        append.append(this.syntaxEscape);
                    }
                    else if (this.isMeta(s, i + this.syntaxEscape.length())) {
                        append.append(this.syntaxEscape);
                    }
                    append.append(this.syntaxEscape);
                    i += this.syntaxEscape.length();
                }
                else if (n && s.startsWith(this.syntaxSeparator, i)) {
                    append.append(this.syntaxEscape).append(this.syntaxSeparator);
                    i += this.syntaxSeparator.length();
                }
                else if (n2 && s.startsWith(this.syntaxSeparator2, i)) {
                    append.append(this.syntaxEscape).append(this.syntaxSeparator2);
                    i += this.syntaxSeparator2.length();
                }
                else {
                    append.append(s.charAt(i++));
                }
            }
        }
        return append.toString();
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        boolean n = true;
        int size = this.components.size();
        String str;
        for (int i = 0; i < size; sb = sb.append(str), ++i) {
            if (this.syntaxDirection == 2) {
                str = this.stringifyComp((String)this.components.elementAt(size - 1 - i));
            }
            else {
                str = this.stringifyComp((String)this.components.elementAt(i));
            }
            if (i != 0 && this.syntaxSeparator != null) {
                sb.append(this.syntaxSeparator);
            }
            if (str.length() >= 1) {
                n = false;
            }
        }
        if (n && size >= 1 && this.syntaxSeparator != null) {
            sb = sb.append(this.syntaxSeparator);
        }
        return sb.toString();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o != null && o instanceof NameImpl) {
            NameImpl nameImpl = (NameImpl)o;
            if (nameImpl.size() == this.size()) {
                Enumeration<String> all = this.getAll();
                Enumeration<String> all2 = nameImpl.getAll();
                while (all.hasMoreElements()) {
                    String trim = (String)all.nextElement();
                    String trim2 = (String)all2.nextElement();
                    if (this.syntaxTrimBlanks) {
                        trim = trim.trim();
                        trim2 = trim2.trim();
                    }
                    if (this.syntaxCaseInsensitive) {
                        if (!trim.equalsIgnoreCase(trim2)) {
                            return false;
                        }
                        continue;
                    }
                    else {
                        if (!trim.equals(trim2)) {
                            return false;
                        }
                        continue;
                    }
                }
                return true;
            }
        }
        return false;
    }
    
    public int compareTo(NameImpl nameImpl) {
        if (this == nameImpl) {
            return 0;
        }
        int size = this.size();
        int size2 = nameImpl.size();
        int min = Math.min(size, size2);
        int n = 0;
        int n2 = 0;
        while (min-- != 0) {
            String s = this.get(n++);
            String s2 = nameImpl.get(n2++);
            if (this.syntaxTrimBlanks) {
                s = s.trim();
                s2 = s2.trim();
            }
            int n3;
            if (this.syntaxCaseInsensitive) {
                n3 = s.compareToIgnoreCase(s2);
            }
            else {
                n3 = s.compareTo(s2);
            }
            if (n3 != 0) {
                return n3;
            }
            continue;
        }
        return size - size2;
    }
    
    public int size() {
        return this.components.size();
    }
    
    public Enumeration<String> getAll() {
        return this.components.elements();
    }
    
    public String get(int index) {
        return (String)this.components.elementAt(index);
    }
    
    public Enumeration<String> getPrefix(int index) {
        if (index < 0 || index > this.size()) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        return new NameImplEnumerator(this.components, 0, index);
    }
    
    public Enumeration<String> getSuffix(int index) {
        int size = this.size();
        if (index < 0 || index > size) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        return new NameImplEnumerator(this.components, index, size);
    }
    
    public boolean isEmpty() {
        return this.components.isEmpty();
    }
    
    public boolean startsWith(int n, Enumeration<String> enumeration) {
        if (n < 0 || n > this.size()) {
            return false;
        }
        try {
            Enumeration<String> prefix = this.getPrefix(n);
            while (prefix.hasMoreElements()) {
                String trim = (String)prefix.nextElement();
                String trim2 = (String)enumeration.nextElement();
                if (this.syntaxTrimBlanks) {
                    trim = trim.trim();
                    trim2 = trim2.trim();
                }
                if (this.syntaxCaseInsensitive) {
                    if (!trim.equalsIgnoreCase(trim2)) {
                        return false;
                    }
                    continue;
                }
                else {
                    if (!trim.equals(trim2)) {
                        return false;
                    }
                    continue;
                }
            }
        }
        catch (NoSuchElementException ex) {
            return false;
        }
        return true;
    }
    
    public boolean endsWith(int n, Enumeration<String> enumeration) {
        int n2 = this.size() - n;
        if (n2 < 0 || n2 > this.size()) {
            return false;
        }
        try {
            Enumeration<String> suffix = this.getSuffix(n2);
            while (suffix.hasMoreElements()) {
                String trim = (String)suffix.nextElement();
                String trim2 = (String)enumeration.nextElement();
                if (this.syntaxTrimBlanks) {
                    trim = trim.trim();
                    trim2 = trim2.trim();
                }
                if (this.syntaxCaseInsensitive) {
                    if (!trim.equalsIgnoreCase(trim2)) {
                        return false;
                    }
                    continue;
                }
                else {
                    if (!trim.equals(trim2)) {
                        return false;
                    }
                    continue;
                }
            }
        }
        catch (NoSuchElementException ex) {
            return false;
        }
        return true;
    }
    
    public boolean addAll(Enumeration<String> enumeration) throws InvalidNameException {
        boolean n = false;
        while (enumeration.hasMoreElements()) {
            try {
                String obj = (String)enumeration.nextElement();
                if (this.size() > 0 && this.syntaxDirection == 0) {
                    throw new InvalidNameException("A flat name can only have a single component");
                }
                this.components.addElement(obj);
                n = true;
            }
            catch (NoSuchElementException ex) {
                break;
            }
        }
        return n;
    }
    
    public boolean addAll(int index, Enumeration<String> enumeration) throws InvalidNameException {
        boolean n = false;
        String obj;
        for (; enumeration.hasMoreElements(); ++index) {
            try {
                obj = (String)enumeration.nextElement();
                if (this.size() > 0 && this.syntaxDirection == 0) {
                    throw new InvalidNameException("A flat name can only have a single component");
                }
                this.components.insertElementAt(obj, index);
                n = true;
            }
            catch (NoSuchElementException ex) {
                break;
            }
        }
        return n;
    }
    
    public void add(String obj) throws InvalidNameException {
        if (this.size() > 0 && this.syntaxDirection == 0) {
            throw new InvalidNameException("A flat name can only have a single component");
        }
        this.components.addElement(obj);
        return;
    }
    
    public void add(int index, String obj) throws InvalidNameException {
        if (this.size() > 0 && this.syntaxDirection == 0) {
            throw new InvalidNameException("A flat name can only zero or one component");
        }
        this.components.insertElementAt(obj, index);
        return;
    }
    
    public Object remove(int n) {
        Object element = this.components.elementAt(n);
        this.components.removeElementAt(n);
        return element;
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        String s;
        for (Enumeration all = this.getAll(); all.hasMoreElements(); n += s.hashCode()) {
            s = (String)all.nextElement();
            if (this.syntaxTrimBlanks) {
                s = s.trim();
            }
            if (this.syntaxCaseInsensitive) {
                s = s.toLowerCase(Locale.ENGLISH);
            }
        }
        return n;
    }
}
